/*
  gFontKB.c: A simple and dumb OnScreen keyboard
  Copyright (c) 2000 C Hanish Menon <hanishkvc@hotmail.com>
  KVCtek, India

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <ctype.h>
#include <gtk/gtk.h>

#include "gtkktoolkit.c"

#define PACKAGE "gfontkb v0.44.2"
#define COPYRIGHT "(c) 2000, C Hanish Menon <hanishkvc@hanishkvc.com>"
#define COMPANY "KVCtek, India"
#define LICENSE "License: GPL"

#define TOTAL_NUMBER_KEYS 256
#define MAX_NUMBER_ROWS 40
#define NUMBER_OF_COLUMNS 20

GtkWidget *windowMain;
GtkWidget *vboxMain, *hboxMain[MAX_NUMBER_ROWS];
GtkWidget *buttonMain[TOTAL_NUMBER_KEYS];
GtkWidget *textMain;
GtkWidget *menubarMain;
GdkFont *fontText;

int iCurRow, iRows, iCurColumn, iColumns, iCurValue;
char sTemp[256];
char sFontName[256];

gint On_WindowDelete(GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
  return FALSE;
}

gint On_ButtonClicked(GtkWidget *widget, gpointer data)
{
  char sTemp[256];

  //if(isalnum(data))
  g_print("%d i.e %c\n",data,data);
  sprintf(sTemp,"%c", data);
  gtk_text_insert(GTK_TEXT(textMain), fontText, NULL, NULL, sTemp, -1);
}

void buttons_remove(void)
{
  for(iCurRow = 0; iCurRow < 256; iCurRow++)
    gtk_widget_destroy(buttonMain[iCurRow]);
  for(iCurRow = 0; iCurRow < iRows; iCurRow++)
    gtk_widget_destroy(hboxMain[iCurRow]);
}

void buttons_create(void)
{
  for(iCurRow = 0; iCurRow < iRows; iCurRow++)
  {
    hboxMain[iCurRow] = gtk_hbox_new(TRUE, TRUE);
    gtk_box_pack_start(GTK_BOX(vboxMain), hboxMain[iCurRow], TRUE, TRUE, 0);
    for(iCurColumn = 0; iCurColumn < iColumns; iCurColumn++)
    {
      iCurValue = iCurRow*iColumns+iCurColumn;
      if(iCurValue >= TOTAL_NUMBER_KEYS)
        break;	
      sprintf(sTemp,"%c",iCurValue);
      buttonMain[iCurValue] = gtk_button_new_with_label(sTemp);
      gtk_box_pack_start(GTK_BOX(hboxMain[iCurRow]), buttonMain[iCurValue],
        TRUE, TRUE, 0);
      gtk_signal_connect(GTK_OBJECT(buttonMain[iCurValue]), "clicked", 
        GTK_SIGNAL_FUNC(On_ButtonClicked), (gpointer)iCurValue );
      gtk_widget_show(buttonMain[iCurValue]);
    }
    gtk_widget_show(hboxMain[iCurRow]);
  }
  
}

/* 
The font selection logic, required for the Multilingual support
*/
void font_selection_ok (GtkWidget *w, GtkFontSelectionDialog *fs)
{
  gchar *s = gtk_font_selection_dialog_get_font_name (fs);
  FILE *fGtkRc;
  char sGtkRc[256];

  if(tmpnam(sGtkRc) == NULL)
    g_error("Unable to generate temporary filename\n");
  if((fGtkRc=fopen(sGtkRc,"w")) == NULL)
    g_error("Error opening temp resource file\n");
  buttons_remove();
  fprintf(fGtkRc,"#Temporary Gtk resource file generated by %s\n", PACKAGE);
  fprintf(fGtkRc,"#If %s is not currently running, then u can safely delete this file\n", PACKAGE);
  fprintf(fGtkRc,"\nstyle \"TextFont_Selected\"\n");
  fprintf(fGtkRc,"{\n font=\"%s\" \n}\n",s);
  fprintf(fGtkRc,
    "widget \"gfontkb.*.GtkButton*\" style \"TextFont_Selected\"\n");
  //fprintf(fGtkRc,"widget_class \"*GtkText\" style \"TextFont_Selected\"\n");
  fclose(fGtkRc);
  gtk_rc_parse(sGtkRc);
  remove(sGtkRc);
  if(s != NULL)
  {
    g_print("Font selected: %s\n", s);
    strcpy(sFontName,s);
    /* Setup the font */
    fontText = gdk_font_load (sFontName);
    gtk_text_insert(GTK_TEXT(textMain), fontText, NULL, NULL, " ", 1);
  }
  else
    g_print("Font not selected\n");
  g_free(s);

  gtk_widget_destroy(GTK_WIDGET (fs));
  buttons_create();
}

void select_font( GtkWidget *widget, gpointer data )
{
  static GtkWidget *window = NULL;

  if(!window)
  {
    window = gtk_font_selection_dialog_new("Font Selection Dialog");
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
      GTK_SIGNAL_FUNC(gtk_widget_destroyed),
      &window);
    gtk_signal_connect(
      GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(window)->ok_button), 
      "clicked", GTK_SIGNAL_FUNC(font_selection_ok),
      GTK_FONT_SELECTION_DIALOG (window));
    gtk_signal_connect_object(
      GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(window)->cancel_button),
      "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
      GTK_OBJECT (window));
  }
  gtk_widget_show (window);
}

void On_MenuItem_Activate(GtkWidget *widget, gpointer data)
{
  if(strcmp((gchar*)data, "Select font") == 0)
  {
    select_font(widget,data); 
  }
  else if(strcmp((gchar*)data,"About") == 0)
  {
    gtkk_message_box_show( PACKAGE"\n"COPYRIGHT"\n"COMPANY"\n"LICENSE);
  }
  else if(strcmp((gchar*)data,"Exit") == 0)
  {
    gtk_main_quit();
  }
  else
  {
    g_message("%s: not yet implemented\n",(gchar*)data);
  }
}

int main(int argc, char **argv)
{

  /* Initialization of global variables, were required */
  iColumns = NUMBER_OF_COLUMNS;
  iRows = TOTAL_NUMBER_KEYS/iColumns + 1;
  fontText = NULL;

  g_print("%s\n",PACKAGE);
  g_print(COPYRIGHT"\n"COMPANY"\n"LICENSE"\n");
  gtk_init(&argc, &argv);
  gtk_rc_init();
  
  windowMain = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name(windowMain,"gfontkb");
  gtk_signal_connect(GTK_OBJECT(windowMain), "delete_event",
    GTK_SIGNAL_FUNC(On_WindowDelete), NULL);
  gtk_window_set_title(GTK_WINDOW(windowMain),PACKAGE COPYRIGHT);
  gtk_container_set_border_width(GTK_CONTAINER(windowMain), 4);
  gtk_widget_set_usize(windowMain, 640, 480);

  vboxMain = gtk_vbox_new(TRUE, 2);
  gtk_container_add(GTK_CONTAINER(windowMain), vboxMain);
  // the Menu bar
  menubarMain = gtk_menu_bar_new();
  gtk_box_pack_start(GTK_BOX(vboxMain), menubarMain, FALSE, TRUE, 0);
  gtkk_menu_new(GTK_MENU_BAR(menubarMain),On_MenuItem_Activate,4,
 	"File",6,"New","Open","Save","SaveAs",GTKK_MENU_SEPERATOR,"Exit",
 	"Edit",4,"Copy","Cut","Paste","SelectAll",
 	"Config",1,"Select font",
 	"Help",1,"About");
  gtk_widget_show(menubarMain);
  // The Text 
  textMain = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(textMain), TRUE);
  gtk_box_pack_start(GTK_BOX(vboxMain), textMain, FALSE, TRUE, 0);
  gtk_widget_show(textMain);

  buttons_create();

  gtk_widget_show(vboxMain);
  gtk_widget_show(GTK_WIDGET(windowMain));
  gtk_main();

  return 0;
}
